/* Copyright (C) 1994 Aladdin Enterprises.  All rights reserved.
  
  This file is part of GNU Ghostscript.
  
  GNU Ghostscript is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY.  No author or distributor accepts responsibility to
  anyone for the consequences of using it or for whether it serves any
  particular purpose or works at all, unless he says so in writing.  Refer
  to the GNU Ghostscript General Public License for full details.
  
*/

/* zfpdiff.c */
/* Pixel-difference filter creation */
#include "memory_.h"
#include "ghost.h"
#include "errors.h"
#include "oper.h"
#include "gsstruct.h"
#include "idict.h"
#include "idparam.h"
#include "strimpl.h"
#include "sfilter.h"
#include "spdiffx.h"
#include "ifilter.h"


/* ------ Color differencing filters ------ */

/* Common setup for encoding and decoding filters. */
private int
diff_setup(os_ptr op, stream_PDiff_state *ss)
{	int code, bpc;
	if ( (code = dict_int_param(op, "Colors", 1, 4, 1,
				    &ss->Colors)) < 0 ||
	     (code = dict_int_param(op, "BitsPerComponent", 1, 8, 8,
				    &bpc)) < 0 ||	     
	     (bpc & (bpc - 1)) != 0 ||
	     (code = dict_int_param(op, "Columns", 0, 9999, -1,
				    &ss->Columns)) < 0
	   )
		return (code < 0 ? code : gs_note_error(e_rangecheck));
	ss->BitsPerComponent = bpc;
	return 0;
}

/* <target> <dict> PixelDifferenceEncode/filter <file> */
private int
zPDiffE(os_ptr op)
{	stream_PDiff_state diffs;
	int code;
	check_type(*op, t_dictionary);
	check_dict_read(*op);
	code = diff_setup(op, &diffs);
	if ( code < 0 )
		return code;
	return filter_write(op, 1, &s_PDiffE_template, (stream_state *)&diffs, 0);
}

/* <source> <dict> PixelDifferenceDecode/filter <file> */
private int
zPDiffD(os_ptr op)
{	stream_PDiff_state diffs;
	int code;
	check_type(*op, t_dictionary);
	check_dict_read(*op);
	code = diff_setup(op, &diffs);
	if ( code < 0 )
		return code;
	return filter_read(op, 1, &s_PDiffD_template, (stream_state *)&diffs, 0);
}

/* ================ Initialization procedure ================ */

BEGIN_OP_DEFS(zfpdiff_op_defs) {
		op_def_begin_filter(),
	{"2PixelDifferenceDecode", zPDiffD},
	{"2PixelDifferenceEncode", zPDiffE},
END_OP_DEFS(0) }
